from indicators_cases import *


def test_np_nan():
    np_nan()


def test_np_ext():
    np_ext()


def test_bias(data):
    bias(data)


def test_macd(data):
    macd(data)


def test_kd(data):
    kd(data)


def test_kdj(data):
    kdj(data)


def test_skd(data):
    skd(data)


def test_amplitude(data):
    amplitude(data)


def test_zig(data):
    zig(data)
